/* ============================================================================
 * Project Name: CReg32-FIFOIn_ex-DMA-RAM_01a
 *
 * Description:
 *   Demo project demonstrating DMA transfer from FIFOIn_ex to the RAM Buffer.
 *   Project uses custom component FIFOIn_ex, providing interface from 
 *   hardware bus and CPU or DMA, implementing 4-word entry FIFO in the UDB.
 *
 *   In this demo, the FIFOIn_ex is configured for 16-bit wide input, with
 *   HalfFullTrigger set to TRUE. When data poulates at least 1/2 of the FIFO,
 *   the "drq" output goes HIGH, forcing DMA transfer into RAM buffer untill
 *   FIFO is empty. Once finished, the Buffer cocntent is printed out
 *   using UART terminal. in this demo the DMA operates in LEVEL mode.
 *    
 * Credits:
 *   FIFOIn component by Brad Budlong (blog space is no longer exist)
 *   This component provides a convenient interface from programmable hardware to
 *   the CPU or DMA.  That interface is implemented using the 4-entry FIFO in the
 *   UDB.
 *   
 * Uses:
 *   FIFOIn_ex - original component modified for a single 16-bit input bus
 *   instead of the 2x8-bit buses
 *
 *   ControlReg32 - custom component (beta version), providing up to 32-bits
 *   output hardware bus. Its performance is similar to the stock ControlRegister
 *   component, but it has no DMA support in >8-bit mode.
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#include <project.h>
#include <stdlib.h>
#include <stdio.h>


#define NO_SAMPLES  16

//============================================
// Global variables
//============================================
char MsgBuf[64];//output UART MsgBufer

int32 counter;

uint16_t Buffer[NO_SAMPLES]; // SRAM


//==============================================================================
// 
//==============================================================================
volatile CYBIT flag_DataReady = 0; 
CY_ISR(myDtrInt) //interrupt on "dtr" rising edge
{   
    flag_DataReady = 1;
}


//============================================
// Forward declarations
//============================================
void DMAConfig(void);
void ReadData(void);
void ReadBuffer(); //read content of the RAM Buffer


void Initialize(void)
{   
   // Clock_STRB_Stop();
    
    CyGlobalIntEnable; //enable global interrupts.
    
    UART_1_Start();         // debugging terminal
    CyDelay(500);
    UART_1_PutString("\nFIFOin demo\r\n\n"); 
    
   
    DMAConfig();
    

    isrDtr_StartEx(myDtrInt);//start isrDtr interrupt

    FIFOIn_1_Start();
    CReg_En_Write(1);
    //Clock_STRB_Start();
}


int main()
{
    Initialize();
    
    counter = -2; // first two data points go for priming FIFOin and will be lost
    
    for(;;) 
    {
         
        if (flag_DataReady==1)  // 
        {
            flag_DataReady=0;
            //ReadData(); // too slow for UART
            
            counter++;
            CReg_Data_Write(counter+250); // set stimulus for next step
            
            
            if (counter == NO_SAMPLES) {
                //isrDtr_Disable();
                //Clock_STRB_Stop();//must stop else continue populate FIFO
                CReg_En_Write(0); // FIFO disable
                ReadBuffer(); 
                return 0;      // terminate program 
            }
              
        }
           
    } //for
    
} //main



//==============================================================================
// 
//==============================================================================

void DMAConfig(void)
{    
    uint8 DMA_Chan;
    uint8 DMA_TD[1];

    #define DMA_BYTES_PER_BURST 4
    #define DMA_REQUEST_PER_BURST 1
    #define DMA_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_DST_BASE (CYDEV_SRAM_BASE)

	DMA_Chan = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
	DMA_TD[0] = CyDmaTdAllocate();
	CyDmaTdSetConfiguration(DMA_TD[0], 2*NO_SAMPLES, DMA_TD[0], TD_TERMIN_EN | TD_INC_DST_ADR);
	CyDmaTdSetAddress(DMA_TD[0], LO16((uint32)FIFOIn_1_FIFO16_PTR), LO16((uint32)(Buffer)));
	CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
	CyDmaChEnable(DMA_Chan, 1);// Enable DMA channel
}


//==============================================================================
// 
//==============================================================================

void ReadData() //read source
{
    sprintf(MsgBuf, "%lu\t %lu\r\n", counter, CReg_Data_Read() ); //
    UART_1_PutString(MsgBuf);              // Send data to PC
}


//==============================================================================
// 
//==============================================================================

void ReadBuffer() //read content of the RAM Buffer
{
    UART_1_PutString("\nRAM Buffer:\r\n\n"); 
    
    uint16 i;
    for(i=0; i<NO_SAMPLES; i++ ) 
    {
        sprintf(MsgBuf, "%d\t %d\r\n", i, Buffer[i] );  
        UART_1_PutString(MsgBuf);              
    }     
}



/// END OF PROGRAM //////////////////////////////////////////////////////////////////////////////////////

